/******************************************************************************
  NTCCompensated.ino

  Marshall Taylor @ SparkFun Electronics

  April 4, 2017

  https://github.com/sparkfun/CCS811_Air_Quality_Breakout
  https://github.com/sparkfun/SparkFun_CCS811_Arduino_Library

  This example uses an NTC thermistor to gather temperature data that is then used
  to compensate the CCS811.  (humidity defaulted at 50%)

  Hardware Connections (Breakoutboard to Arduino):
  3.3V to 3.3V pin
  GND to GND pin
  SDA to A4
  SCL to A5
  SEN-00250 (NTCLE100E3103JB0) between NTC terminals

  Resources:
  Uses Wire.h for i2c operation

  Development environment specifics:
  Arduino IDE 1.8.1

  This code is released under the [MIT License](http://opensource.org/licenses/MIT).

  Please review the LICENSE.md file included with this example. If you have any questions
  or concerns with licensing, please contact techsupport@sparkfun.com.

  Distributed as-is; no warranty is given.
******************************************************************************/
#include <SparkFunCCS811.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1331.h>
#include <SPI.h>

#define CCS811_ADDR 0x5B //Default I2C Address
//#define CCS811_ADDR 0x5A //Alternate I2C Address

CCS811 myCCS811(CCS811_ADDR);


/*
//FOR ARDUINO UNO
#define sclk 13
#define mosi 11
#define cs   10
#define rst  9
#define dc   8
*/

//FOR ARDUINO MICRO
#define sclk 9
#define mosi 8
#define cs   10
#define rst  12
#define dc   11



// Color definitions
#define  BLACK           0x0000
#define BLUE            0x001F
#define RED             0xF800
#define GREEN           0x07E0
#define CYAN            0x07FF
#define MAGENTA         0xF81F
#define YELLOW          0xFFE0  
#define WHITE           0xFFFF

int CO2;
int TVOC;
float TEMP;
float HUMIDITY;
int LEVEL;
int prec_CO2;
int prec_TVOC;
float prec_TEMP;
int prec_LEVEL = 1;
int completo = 0;

Adafruit_SSD1331 display = Adafruit_SSD1331(cs, dc, mosi, sclk, rst);  


void setup()
{
  display.begin();
  uint16_t time = millis();
  display.fillScreen(BLACK);
  time = millis() - time;
  
  display.setTextColor(WHITE);
  display.fillScreen(BLACK);
  display.setTextSize(1);
  display.setCursor(30,0);
  display.println("CCS811");
  display.setCursor(18,15);
  display.println("Indoor Air");
  display.setCursor(6,30);
  display.println("Quality Sensor");
  display.setCursor(30,50);
  display.println("v. 1.0");
  delay(1000);


  Serial.begin(9600);
  Serial.println();
  Serial.println("Apply NTC data to CCS811 for compensation.");
  CCS811Core::status returnCode = myCCS811.begin();
  Serial.print("CCS811 begin exited with: ");
  printDriverError( returnCode );
  Serial.println();
  myCCS811.setRefResistance( 100000 );
  delay(1000);
}

void loop()
{
  if (myCCS811.dataAvailable())
  {
    myCCS811.readAlgorithmResults(); //Calling this function updates the global tVOC and CO2 variables
    CO2= myCCS811.getCO2();
    TVOC= myCCS811.getTVOC();
    myCCS811.readNTC();
    TEMP= myCCS811.getTemperature();
    HUMIDITY=50;
    
    Serial.print("CO2: ");
    Serial.print(CO2);
    Serial.print(" ppm");
    Serial.print("   -   ");
        
    Serial.print("TVOC: ");
    Serial.print(TVOC);
    Serial.print(" ppb");
    Serial.print("   -   ");

    Serial.print("T: ");    
    Serial.print( TEMP, 2);
    Serial.println(" °C");

    //Pass the temperature back into the CCS811 to compensate
    myCCS811.setEnvironmentalData( HUMIDITY, TEMP);

    PrintDataOLED(CO2,TVOC,TEMP);
    delay (1000);
  }
  else if (myCCS811.checkForStatusError())
  {
    printSensorError();
  }

  delay(10); //Don't spam the I2C bus
}

//printDriverError decodes the CCS811Core::status type and prints the
//type of error to the serial terminal.
//
//Save the return value of any function of type CCS811Core::status, then pass
//to this function to see what the output was.
void printDriverError( CCS811Core::status errorCode )
{
  display.fillScreen(BLACK);
  display.setTextColor(RED);
  display.setCursor(5, 30);  
  switch ( errorCode )
  {
    case CCS811Core::SENSOR_SUCCESS:
      Serial.print("SUCCESS");
      display.print("SENSOR DETECT");
      break;
    case CCS811Core::SENSOR_ID_ERROR:
      Serial.print("ID_ERROR");
      display.print("ID_ERROR");
      break;
    case CCS811Core::SENSOR_I2C_ERROR:
      Serial.print("I2C_ERROR");
      display.print("I2C_ERROR");
      break;
    case CCS811Core::SENSOR_INTERNAL_ERROR:
      Serial.print("INTERNAL_ERROR");
      display.print("INTERNAL_ERROR");
      break;
    case CCS811Core::SENSOR_GENERIC_ERROR:
      Serial.print("GENERIC_ERROR");
      display.print("GENERIC_ERROR");
      break;
    default:
      Serial.print("Unspecified error.");
      display.print("ERROR");
  }
}

//printSensorError gets, clears, then prints the errors
//saved within the error register.
void printSensorError()
{
  uint8_t error = myCCS811.getErrorRegister();

  if ( error == 0xFF ) //comm error
  {
    Serial.println("Failed to get ERROR_ID register.");
  }
  else
  {
    Serial.print("Error: ");
    if (error & 1 << 5) Serial.print("HeaterSupply");
    if (error & 1 << 4) Serial.print("HeaterFault");
    if (error & 1 << 3) Serial.print("MaxResistance");
    if (error & 1 << 2) Serial.print("MeasModeInvalid");
    if (error & 1 << 1) Serial.print("ReadRegInvalid");
    if (error & 1 << 0) Serial.print("MsgInvalid");
    Serial.println();
  }
}



void PrintDataOLED(int data1, int data2, float data3) 
{ 
  display.setTextColor(WHITE);
  
  display.setTextSize(1); 
  if (completo==0)  //Visualizzo tutti i dettagli al display se è la prima volta che si visualizzano
  {
    completo=1;
    display.fillScreen(BLACK);
    display.setCursor(0, 0);
    display.print("Air Quality");
    display.drawLine(0,10,96,10,WHITE);
    
    display.setCursor(0, 15);
    display.print("CO2: ");

    display.setCursor(0, 28);
    display.print("TVOC: ");

    display.setCursor(0, 41);
    display.print("Temp: ");
  }
   //Si visualizzano solo i valori cancellando quelli precedentemente scritti
  //CO2
  display.setTextColor(BLACK);
  display.setCursor(33, 15);
  display.print(prec_CO2);
  display.print(" ppm");
  if (data1<=1000)
  {
    display.setTextColor(GREEN);   
  }
  if ((data1>1000)&&(data1<=1500))
  {
    display.setTextColor(YELLOW);   
  }
  if (data1>1500)
  {
    display.setTextColor(RED);   
  }  
  display.setCursor(33, 15);
  display.print(data1);
  display.print(" ppm");

  //TVOC
  display.setTextColor(BLACK);
  display.setCursor(33, 28);
  display.print(prec_TVOC);
  display.print(" ppb");
  if (data2<=50)
  {
    display.setTextColor(GREEN);   
  }
  if ((data2>50)&&(data2<=150))
  {
    display.setTextColor(YELLOW);   
  }
  if (data2>150)
  {
    display.setTextColor(RED);   
  }
  display.setCursor(33, 28);
  display.print(data2);
  display.print(" ppb");

  //Temperatura
  display.setTextColor(BLACK);
  display.setCursor(33, 41);
  display.print(prec_TEMP);
  display.print(" C");
  if (data3<=15)
  {
    display.setTextColor(BLUE);   
  }
  if ((data3>15)&&(data3<=30))
  {
    display.setTextColor(YELLOW);   
  }
  if (data3>30)
  {
    display.setTextColor(RED);   
  }
  display.setCursor(33, 41);
  display.print(data3);
  display.print(" C");

  Print_Level(data1,prec_LEVEL);
  
  prec_CO2=data1;
  prec_TVOC=data2;
  prec_TEMP=data3;
}


void Print_Level(int data1, int &prec_data1)
{
  if (data1<=600)
  {
    LEVEL=1;
  }
  if ((data1>600)&&(data1<=800))
  {
    LEVEL=2;
  }
  if ((data1>800)&&(data1<=1000))
  {
    LEVEL=3;
  }
  if ((data1>1000)&&(data1<=1500))
  {
    LEVEL=4;
  }
  if (data1>1500)
  {
    LEVEL=5;
  }
  
  for (int i=0;i<=1;i++)
  {
    if (i==0) 
    {
      data1=prec_data1;
      display.setTextColor(BLACK);  
    }
    if (i==1) 
    {
      prec_data1=LEVEL; 
      data1=LEVEL;
      switch (data1)
      {
        case 1:
          display.setTextColor(GREEN); 
          break;
        case 2:
          display.setTextColor(GREEN);
          break;
        case 3:
          display.setTextColor(GREEN); 
          break;
        case 4:
          display.setTextColor(YELLOW); 
          break;
        case 5:
          display.setTextColor(RED); 
          break;
      }
    }
    display.setCursor(0, 54);
    switch (data1)
    {
      case 1:
        display.print("Excellent Air");
        break;
      case 2:
        display.print("Good Air");
        break;
      case 3:
        display.print("Fair Air");
        break;
      case 4:
        display.print("Mediocre Air");
        break;
      case 5: 
        display.print("Bad Air");
        break;
    }
  }    
}

